aboutsummaryrefslogtreecommitdiff
path: root/share/rpcauth/rpcauth.py
diff options
context:
space:
mode:
Diffstat (limited to 'share/rpcauth/rpcauth.py')
-rwxr-xr-xshare/rpcauth/rpcauth.py47
1 files changed, 23 insertions, 24 deletions
diff --git a/share/rpcauth/rpcauth.py b/share/rpcauth/rpcauth.py
index 13bef3d37a..b14c80171e 100755
--- a/share/rpcauth/rpcauth.py
+++ b/share/rpcauth/rpcauth.py
@@ -3,45 +3,44 @@
# Distributed under the MIT software license, see the accompanying
# file COPYING or http://www.opensource.org/licenses/mit-license.php.
-import sys
-import os
-from random import SystemRandom
-import base64
-import hmac
+from argparse import ArgumentParser
+from base64 import urlsafe_b64encode
+from binascii import hexlify
+from getpass import getpass
+from os import urandom
-def generate_salt():
- # This uses os.urandom() underneath
- cryptogen = SystemRandom()
+import hmac
- # Create 16 byte hex salt
- salt_sequence = [cryptogen.randrange(256) for _ in range(16)]
- return ''.join([format(r, 'x') for r in salt_sequence])
+def generate_salt(size):
+ """Create size byte hex salt"""
+ return hexlify(urandom(size)).decode()
def generate_password():
"""Create 32 byte b64 password"""
- return base64.urlsafe_b64encode(os.urandom(32)).decode('utf-8')
+ return urlsafe_b64encode(urandom(32)).decode('utf-8')
def password_to_hmac(salt, password):
m = hmac.new(bytearray(salt, 'utf-8'), bytearray(password, 'utf-8'), 'SHA256')
return m.hexdigest()
def main():
- if len(sys.argv) < 2:
- sys.stderr.write('Please include username (and an optional password, will generate one if not provided) as an argument.\n')
- sys.exit(0)
+ parser = ArgumentParser(description='Create login credentials for a JSON-RPC user')
+ parser.add_argument('username', help='the username for authentication')
+ parser.add_argument('password', help='leave empty to generate a random password or specify "-" to prompt for password', nargs='?')
+ args = parser.parse_args()
- username = sys.argv[1]
+ if not args.password:
+ args.password = generate_password()
+ elif args.password == '-':
+ args.password = getpass()
- salt = generate_salt()
- if len(sys.argv) > 2:
- password = sys.argv[2]
- else:
- password = generate_password()
- password_hmac = password_to_hmac(salt, password)
+ # Create 16 byte hex salt
+ salt = generate_salt(16)
+ password_hmac = password_to_hmac(salt, args.password)
print('String to be appended to bitcoin.conf:')
- print('rpcauth={0}:{1}${2}'.format(username, salt, password_hmac))
- print('Your password:\n{0}'.format(password))
+ print('rpcauth={0}:{1}${2}'.format(args.username, salt, password_hmac))
+ print('Your password:\n{0}'.format(args.password))
if __name__ == '__main__':
main()